function [] = strengthga( overallfitness )
%STRENGTHGA Genetischer Algorithmus zur Anpassung der Fitnesswerte
%   Dieser GA soll nach mehreren erfolgten Simulationsläufen mit dem
%   gleichen Regelsatz aber unterschiedlichen Fitnesswerten der einzelnen
%   Regeln die Ergebnisse der Läufe miteinander vergleichen und aufgrund
%   der globalen Fitness der entstandenen Strukturen die einzelnen
%   Stärkewerte der Regeln genetisch rekombinieren. Dadurch sollen neue
%   Regelsätze erzeugt werden, die sich nur durch die Fitness der einzelnen
%   Regeln von den alten Regelsätzen unterscheiden.

global wespe;
global Wabengitter;
global MittelWabe;

probmutation = 0.2; % feste Mutationswahrscheinlichkeit

overallfitness = overallfitness / sum(overallfitness); % Normierung auf Gesamtfitness 1
                     % wichtig für Roulette_wheel-Selection

cumulfitness = 0;
for regelsatz=1:length(overallfitness)
    maxfitness(regelsatz) = cumulfitness + overallfitness(regelsatz);
end

% Selektion
eltern = roulette(overallfitness);

% Rekombination
newstrings = onepointxover(eltern);

% Mutation
newstrings = mutation(newstrings);

% Erzeugen von zwei zusätzlichen bauagent-Objekten in wespe
rulesetanzahl = size(wespe,1);
wespe(rulesetanzahl+1, 1) = bauagent(Wabengitter(1:2, MittelWabe)', 1);
wespe(rulesetanzahl+2, 1) = bauagent(Wabengitter(1:2, MittelWabe)', 1);

% Einfügen der neuen Fitnessstrings
wespe(rulesetanzahl+1,1).classifierfitnessstring = newstrings(1,:);
wespe(rulesetanzahl+1,1).updatefitness;
wespe(rulesetanzahl+2,1).classifierfitnessstring = newstrings(2,:);
wespe(rulesetanzahl+2,1).updatefitness;

function index = roulette(overallfitness)
    a = rand;
    b = rand;
    
    for ruleset=length(overallfitness):-1:2
        if ( a > maxfitness(ruleset-1) )
            index(1) = ruleset;
            % maxfitness(ruleset-1) = 1.0; % verhindert, dass für index2 der gleiche Wert gewählt wird
            break;
        end
    end
    if ( a <= maxfitness(1) )
        index(1) = 1;
    end
    
    for ruleset=length(overallfitness):-1:2
        if ( b > maxfitness(ruleset-1) )
            index(2) = ruleset;
            break;
        end
    end
    if ( b <= maxfitness(1) )
        index(2) = 1;
    end
    % TODO: index1 und index2 müssen verschieden sein!  
end

    function kinder = onepointxover(eltern)
        string1 = wespe(eltern(1),1).classifierfitnessstring;
        string2 = wespe(eltern(2),1).classifierfitnessstring;
           % kopiert die Fitness-Strings der ausgewählten Eltern in Graycode
           % in zwei lokale String-Variablen
           % Es wird der Gesamtstring (Stärke aller Regeln
           % hintereinander) verwendet
        xoverpoint = uint16(rand * length(string1));
           % wählt einen zufälligen Crossoverpunkt
        neustring1 = [string1(1:xoverpoint) string2(xoverpoint+1:length(string2))];
        neustring2 = [string2(1:xoverpoint) string1(xoverpoint+1:length(string1))];
        
        kinder = [neustring1; neustring2];
    end
    
    function newbitstring = mutation(bitstring)
        bitstring1 = bitstring(1,:);
        bitstring2 = bitstring(2,:);
        
        if (rand < probmutation) % wenn Zufallszahl unter Mutationswahrscheinlichkeit
            mutbit = uint16(rand * length(bitstring1));
            if strcmp(bitstring1(mutbit),'0')
                bitstring1(mutbit) = '1';
            end
            if strcmp(bitstring1(mutbit),'1')
                bitstring1(mutbit) = '0';
            end
        end % für bitstring1
        
        if (rand < probmutation) % wenn Zufallszahl unter Mutationswahrscheinlichkeit
            mutbit = uint16(rand * length(bitstring2));
            if strcmp(bitstring2(mutbit),'0')
                bitstring2(mutbit) = '1';
            end
            if strcmp(bitstring2(mutbit),'1')
                bitstring2(mutbit) = '0';
            end
        end % für bitstring2
        
        newbitstring = [bitstring1; bitstring2];
    end
        
end